@@ -1,23 +1,15 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2015, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   User interface for the direct solver object in SLEPc
+*/
 
 #if !defined(__SLEPCDS_H)
 #define __SLEPCDS_H
@@ -47,14 +39,14 @@ typedef struct _p_DS* DS;
 .seealso: DSSetType(), DS
 J*/
 typedef const char* DSType;
-#define DSHEP             "hep"
-#define DSNHEP            "nhep"
-#define DSGHEP            "ghep"
-#define DSGHIEP           "ghiep"
-#define DSGNHEP           "gnhep"
-#define DSSVD             "svd"
-#define DSPEP             "pep"
-#define DSNEP             "nep"
+#define DSHEP    "hep"
+#define DSNHEP   "nhep"
+#define DSGHEP   "ghep"
+#define DSGHIEP  "ghiep"
+#define DSGNHEP  "gnhep"
+#define DSSVD    "svd"
+#define DSPEP    "pep"
+#define DSNEP    "nep"
 
 /* Logging support */
 PETSC_EXTERN PetscClassId DS_CLASSID;
@@ -70,6 +62,7 @@ typedef enum { DS_STATE_RAW,
                DS_STATE_INTERMEDIATE,
                DS_STATE_CONDENSED,
                DS_STATE_TRUNCATED } DSStateType;
+PETSC_EXTERN const char *DSStateTypes[];
 
 /*E
     DSMatType - Used to refer to one of the matrices stored internally in DS
@@ -129,6 +122,17 @@ typedef enum { DS_MAT_A,
 PETSC_EXTERN DSMatType DSMatExtra[];
 #define DS_NUM_EXTRA  10
 
+/*E
+    DSParallelType - Indicates the parallel mode that the direct solver will use
+
+    Level: advanced
+
+.seealso: DSSetParallel()
+E*/
+typedef enum { DS_PARALLEL_REDUNDANT,
+               DS_PARALLEL_SYNCHRONIZED } DSParallelType;
+PETSC_EXTERN const char *DSParallelTypes[];
+
 PETSC_EXTERN PetscErrorCode DSCreate(MPI_Comm,DS*);
 PETSC_EXTERN PetscErrorCode DSSetType(DS,DSType);
 PETSC_EXTERN PetscErrorCode DSGetType(DS,DSType*);
@@ -153,6 +157,8 @@ PETSC_EXTERN PetscErrorCode DSTruncate(DS,PetscInt);
 PETSC_EXTERN PetscErrorCode DSSetIdentity(DS,DSMatType);
 PETSC_EXTERN PetscErrorCode DSSetMethod(DS,PetscInt);
 PETSC_EXTERN PetscErrorCode DSGetMethod(DS,PetscInt*);
+PETSC_EXTERN PetscErrorCode DSSetParallel(DS,DSParallelType);
+PETSC_EXTERN PetscErrorCode DSGetParallel(DS,DSParallelType*);
 PETSC_EXTERN PetscErrorCode DSSetCompact(DS,PetscBool);
 PETSC_EXTERN PetscErrorCode DSGetCompact(DS,PetscBool*);
 PETSC_EXTERN PetscErrorCode DSSetExtraRow(DS,PetscBool);
@@ -168,23 +174,30 @@ PETSC_EXTERN PetscErrorCode DSRestoreArrayReal(DS,DSMatType,PetscReal*[]);
 PETSC_EXTERN PetscErrorCode DSVectors(DS,DSMatType,PetscInt*,PetscReal*);
 PETSC_EXTERN PetscErrorCode DSSolve(DS,PetscScalar*,PetscScalar*);
 PETSC_EXTERN PetscErrorCode DSSort(DS,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*,PetscInt*);
+PETSC_EXTERN PetscErrorCode DSSynchronize(DS,PetscScalar*,PetscScalar*);
 PETSC_EXTERN PetscErrorCode DSCopyMat(DS,DSMatType,PetscInt,PetscInt,Mat,PetscInt,PetscInt,PetscInt,PetscInt,PetscBool);
+PETSC_EXTERN PetscErrorCode DSMatGetSize(DS,DSMatType,PetscInt*,PetscInt*);
+PETSC_EXTERN PetscErrorCode DSMatIsHermitian(DS,DSMatType,PetscBool*);
 PETSC_EXTERN PetscErrorCode DSSetSlepcSC(DS,SlepcSC);
 PETSC_EXTERN PetscErrorCode DSGetSlepcSC(DS,SlepcSC*);
 PETSC_EXTERN PetscErrorCode DSUpdateExtraRow(DS);
 PETSC_EXTERN PetscErrorCode DSCond(DS,PetscReal*);
 PETSC_EXTERN PetscErrorCode DSTranslateHarmonic(DS,PetscScalar,PetscReal,PetscBool,PetscScalar*,PetscReal*);
 PETSC_EXTERN PetscErrorCode DSTranslateRKS(DS,PetscScalar);
-PETSC_EXTERN PetscErrorCode DSNormalize(DS,DSMatType,PetscInt);
+PETSC_EXTERN PetscErrorCode DSOrthogonalize(DS,DSMatType,PetscInt,PetscInt*);
+PETSC_EXTERN PetscErrorCode DSPseudoOrthogonalize(DS,DSMatType,PetscInt,PetscReal*,PetscInt*,PetscReal*);
 
 /* --------- options specific to particular solvers -------- */
 
 PETSC_EXTERN PetscErrorCode DSPEPSetDegree(DS,PetscInt);
 PETSC_EXTERN PetscErrorCode DSPEPGetDegree(DS,PetscInt*);
+PETSC_EXTERN PetscErrorCode DSPEPSetCoefficients(DS,PetscReal*);
+PETSC_EXTERN PetscErrorCode DSPEPGetCoefficients(DS,PetscReal**);
 
 PETSC_EXTERN PetscErrorCode DSNEPSetFN(DS,PetscInt,FN*);
 PETSC_EXTERN PetscErrorCode DSNEPGetFN(DS,PetscInt,FN*);
 PETSC_EXTERN PetscErrorCode DSNEPGetNumFN(DS,PetscInt*);
+PETSC_EXTERN PetscErrorCode DSNEPSetComputeMatrixFunction(DS,PetscErrorCode (*fun)(DS,PetscScalar,PetscBool,DSMatType,void*),void *ctx);
 
 PETSC_EXTERN PetscFunctionList DSList;
 PETSC_EXTERN PetscErrorCode DSRegister(const char[],PetscErrorCode(*)(DS));
