@@ -1,25 +1,15 @@
 /*
-   Spectral transformation module for eigenvalue problems.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2015, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Spectral transformation module for eigenvalue problems
+*/
 
 #if !defined(__SLEPCST_H)
 #define __SLEPCST_H
@@ -52,6 +42,7 @@ typedef const char* STType;
 #define STSINVERT   "sinvert"
 #define STCAYLEY    "cayley"
 #define STPRECOND   "precond"
+#define STFILTER    "filter"
 
 /* Logging support */
 PETSC_EXTERN PetscClassId ST_CLASSID;
@@ -61,14 +52,26 @@ PETSC_EXTERN PetscErrorCode STDestroy(ST*);
 PETSC_EXTERN PetscErrorCode STReset(ST);
 PETSC_EXTERN PetscErrorCode STSetType(ST,STType);
 PETSC_EXTERN PetscErrorCode STGetType(ST,STType*);
-PETSC_EXTERN PetscErrorCode STSetOperators(ST,PetscInt,Mat*);
-PETSC_EXTERN PetscErrorCode STGetOperators(ST,PetscInt,Mat*);
-PETSC_EXTERN PetscErrorCode STGetTOperators(ST,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode STSetMatrices(ST,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode STGetMatrix(ST,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode STGetMatrixTransformed(ST,PetscInt,Mat*);
 PETSC_EXTERN PetscErrorCode STGetNumMatrices(ST,PetscInt*);
+PETSC_EXTERN PetscErrorCode STGetOperator(ST,Mat*);
 PETSC_EXTERN PetscErrorCode STSetUp(ST);
 PETSC_EXTERN PetscErrorCode STSetFromOptions(ST);
 PETSC_EXTERN PetscErrorCode STView(ST,PetscViewer);
 
+PETSC_DEPRECATED("Use STSetMatrices()") PETSC_STATIC_INLINE PetscErrorCode STSetOperators(ST st,PetscInt n,Mat *A) {return STSetMatrices(st,n,A);}
+PETSC_DEPRECATED("Use STGetMatrix()") PETSC_STATIC_INLINE PetscErrorCode STGetOperators(ST st,PetscInt k,Mat *A) {return STGetMatrix(st,k,A);}
+PETSC_DEPRECATED("Use STGetMatrixTransformed()") PETSC_STATIC_INLINE PetscErrorCode STGetTOperators(ST st,PetscInt k,Mat *A) {return STGetMatrixTransformed(st,k,A);}
+PETSC_DEPRECATED("Use STGetOperator() followed by MatComputeExplicitOperator()") PETSC_STATIC_INLINE PetscErrorCode STComputeExplicitOperator(ST st,Mat *A) {
+  PetscErrorCode ierr; Mat Op; 
+  ierr = STGetOperator(st,&Op);CHKERRQ(ierr);
+  ierr = MatComputeExplicitOperator(Op,A);CHKERRQ(ierr);
+  ierr = MatDestroy(&Op);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PETSC_EXTERN PetscErrorCode STApply(ST,Vec,Vec);
 PETSC_EXTERN PetscErrorCode STMatMult(ST,PetscInt,Vec,Vec);
 PETSC_EXTERN PetscErrorCode STMatMultTranspose(ST,PetscInt,Vec,Vec);
@@ -76,9 +79,10 @@ PETSC_EXTERN PetscErrorCode STMatSolve(ST,Vec,Vec);
 PETSC_EXTERN PetscErrorCode STMatSolveTranspose(ST,Vec,Vec);
 PETSC_EXTERN PetscErrorCode STGetBilinearForm(ST,Mat*);
 PETSC_EXTERN PetscErrorCode STApplyTranspose(ST,Vec,Vec);
-PETSC_EXTERN PetscErrorCode STComputeExplicitOperator(ST,Mat*);
 PETSC_EXTERN PetscErrorCode STMatSetUp(ST,PetscScalar,PetscScalar*);
 PETSC_EXTERN PetscErrorCode STPostSolve(ST);
+PETSC_EXTERN PetscErrorCode STResetMatrixState(ST);
+PETSC_EXTERN PetscErrorCode STSetWorkVecs(ST,PetscInt);
 
 PETSC_EXTERN PetscErrorCode STSetKSP(ST,KSP);
 PETSC_EXTERN PetscErrorCode STGetKSP(ST,KSP*);
@@ -96,10 +100,12 @@ PETSC_EXTERN PetscErrorCode STAppendOptionsPrefix(ST,const char*);
 PETSC_EXTERN PetscErrorCode STGetOptionsPrefix(ST,const char*[]);
 
 PETSC_EXTERN PetscErrorCode STBackTransform(ST,PetscInt,PetscScalar*,PetscScalar*);
+PETSC_EXTERN PetscErrorCode STIsInjective(ST,PetscBool*);
 
 PETSC_EXTERN PetscErrorCode STCheckNullSpace(ST,BV);
 
 PETSC_EXTERN PetscErrorCode STMatCreateVecs(ST,Vec*,Vec*);
+PETSC_EXTERN PetscErrorCode STMatCreateVecsEmpty(ST,Vec*,Vec*);
 PETSC_EXTERN PetscErrorCode STMatGetSize(ST,PetscInt*,PetscInt*);
 PETSC_EXTERN PetscErrorCode STMatGetLocalSize(ST,PetscInt*,PetscInt*);
 
@@ -114,6 +120,8 @@ E*/
 typedef enum { ST_MATMODE_COPY,
                ST_MATMODE_INPLACE,
                ST_MATMODE_SHELL } STMatMode;
+PETSC_EXTERN const char *STMatModes[];
+
 PETSC_EXTERN PetscErrorCode STSetMatMode(ST,STMatMode);
 PETSC_EXTERN PetscErrorCode STGetMatMode(ST,STMatMode*);
 PETSC_EXTERN PetscErrorCode STSetMatStructure(ST,MatStructure);
@@ -138,5 +146,13 @@ PETSC_EXTERN PetscErrorCode STPrecondSetMatForPC(ST,Mat);
 PETSC_EXTERN PetscErrorCode STPrecondGetKSPHasMat(ST,PetscBool*);
 PETSC_EXTERN PetscErrorCode STPrecondSetKSPHasMat(ST,PetscBool);
 
+PETSC_EXTERN PetscErrorCode STFilterSetInterval(ST,PetscReal,PetscReal);
+PETSC_EXTERN PetscErrorCode STFilterGetInterval(ST,PetscReal*,PetscReal*);
+PETSC_EXTERN PetscErrorCode STFilterSetRange(ST,PetscReal,PetscReal);
+PETSC_EXTERN PetscErrorCode STFilterGetRange(ST,PetscReal*,PetscReal*);
+PETSC_EXTERN PetscErrorCode STFilterSetDegree(ST,PetscInt);
+PETSC_EXTERN PetscErrorCode STFilterGetDegree(ST,PetscInt*);
+PETSC_EXTERN PetscErrorCode STFilterGetThreshold(ST,PetscReal*);
+
 #endif
 
