@@ -1,34 +1,28 @@
 /*
-   BV (basis vectors) interface routines, callable by users.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2015, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   BV (basis vectors) interface routines, callable by users
+*/
 
 #include <slepc/private/bvimpl.h>            /*I "slepcbv.h" I*/
 
 PetscClassId     BV_CLASSID = 0;
-PetscLogEvent    BV_Create = 0,BV_Copy = 0,BV_Mult = 0,BV_Dot = 0,BV_Orthogonalize = 0,BV_Scale = 0,BV_Norm = 0,BV_SetRandom = 0,BV_MatMult = 0,BV_MatProject = 0,BV_AXPY = 0;
+PetscLogEvent    BV_Create = 0,BV_Copy = 0,BV_Mult = 0,BV_MultVec = 0,BV_MultInPlace = 0,BV_Dot = 0,BV_DotVec = 0,BV_Orthogonalize = 0,BV_OrthogonalizeVec = 0,BV_Scale = 0,BV_Norm = 0,BV_NormVec = 0,BV_SetRandom = 0,BV_MatMult = 0,BV_MatMultVec = 0,BV_MatProject = 0;
 static PetscBool BVPackageInitialized = PETSC_FALSE;
+MPI_Op MPIU_TSQR = 0;
+
+const char *BVOrthogTypes[] = {"CGS","MGS","BVOrthogType","BV_ORTHOG_",0};
+const char *BVOrthogRefineTypes[] = {"IFNEEDED","NEVER","ALWAYS","BVOrthogRefineType","BV_ORTHOG_REFINE_",0};
+const char *BVOrthogBlockTypes[] = {"GS","CHOL","TSQR","TSQRCHOL","SVQB","BVOrthogBlockType","BV_ORTHOG_BLOCK_",0};
+const char *BVMatMultTypes[] = {"VECS","MAT","MAT_SAVE","BVMatMultType","BV_MATMULT_",0};
 
-#undef __FUNCT__
-#define __FUNCT__ "BVFinalizePackage"
 /*@C
    BVFinalizePackage - This function destroys everything in the Slepc interface
    to the BV package. It is called from SlepcFinalize().
@@ -43,13 +37,12 @@ PetscErrorCode BVFinalizePackage(void)
 
   PetscFunctionBegin;
   ierr = PetscFunctionListDestroy(&BVList);CHKERRQ(ierr);
+  ierr = MPI_Op_free(&MPIU_TSQR);CHKERRQ(ierr);
   BVPackageInitialized = PETSC_FALSE;
   BVRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVInitializePackage"
 /*@C
    BVInitializePackage - This function initializes everything in the BV package.
    It is called from PetscDLLibraryRegister() when using dynamic libraries, and
@@ -62,8 +55,7 @@ PetscErrorCode BVFinalizePackage(void)
 PetscErrorCode BVInitializePackage(void)
 {
   char           logList[256];
-  char           *className;
-  PetscBool      opt;
+  PetscBool      opt,pkg;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -77,36 +69,38 @@ PetscErrorCode BVInitializePackage(void)
   ierr = PetscLogEventRegister("BVCreate",BV_CLASSID,&BV_Create);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVCopy",BV_CLASSID,&BV_Copy);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVMult",BV_CLASSID,&BV_Mult);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("BVMultVec",BV_CLASSID,&BV_MultVec);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("BVMultInPlace",BV_CLASSID,&BV_MultInPlace);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVDot",BV_CLASSID,&BV_Dot);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("BVDotVec",BV_CLASSID,&BV_DotVec);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVOrthogonalize",BV_CLASSID,&BV_Orthogonalize);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("BVOrthogonalizeV",BV_CLASSID,&BV_OrthogonalizeVec);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVScale",BV_CLASSID,&BV_Scale);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVNorm",BV_CLASSID,&BV_Norm);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("BVNormVec",BV_CLASSID,&BV_NormVec);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVSetRandom",BV_CLASSID,&BV_SetRandom);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVMatMult",BV_CLASSID,&BV_MatMult);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("BVMatMultVec",BV_CLASSID,&BV_MatMultVec);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BVMatProject",BV_CLASSID,&BV_MatProject);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("BVAXPY",BV_CLASSID,&BV_AXPY);CHKERRQ(ierr);
+  /* MPI reduction operation used in BVOrthogonalize */
+  ierr = MPI_Op_create(SlepcGivensPacked,PETSC_TRUE,&MPIU_TSQR);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"bv",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(BV_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("bv",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscInfoDeactivateClass(BV_CLASSID);CHKERRQ(ierr); }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"bv",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(BV_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("bv",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscLogEventDeactivateClass(BV_CLASSID);CHKERRQ(ierr); }
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(BVFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVDestroy"
 /*@
    BVDestroy - Destroys BV context that was created with BVCreate().
 
@@ -126,23 +120,27 @@ PetscErrorCode BVDestroy(BV *bv)
   PetscFunctionBegin;
   if (!*bv) PetscFunctionReturn(0);
   PetscValidHeaderSpecific(*bv,BV_CLASSID,1);
+  if ((*bv)->lsplit) SETERRQ(PetscObjectComm((PetscObject)(*bv)),PETSC_ERR_ARG_WRONGSTATE,"Must call BVRestoreSplit before destroying the BV");
   if (--((PetscObject)(*bv))->refct > 0) { *bv = 0; PetscFunctionReturn(0); }
   if ((*bv)->ops->destroy) { ierr = (*(*bv)->ops->destroy)(*bv);CHKERRQ(ierr); }
   ierr = VecDestroy(&(*bv)->t);CHKERRQ(ierr);
   ierr = MatDestroy(&(*bv)->matrix);CHKERRQ(ierr);
   ierr = VecDestroy(&(*bv)->Bx);CHKERRQ(ierr);
+  ierr = VecDestroy(&(*bv)->buffer);CHKERRQ(ierr);
   ierr = BVDestroy(&(*bv)->cached);CHKERRQ(ierr);
+  ierr = BVDestroy(&(*bv)->L);CHKERRQ(ierr);
+  ierr = BVDestroy(&(*bv)->R);CHKERRQ(ierr);
   ierr = PetscFree((*bv)->work);CHKERRQ(ierr);
   ierr = PetscFree2((*bv)->h,(*bv)->c);CHKERRQ(ierr);
-  ierr = PetscFree((*bv)->omega);CHKERRQ(ierr);
-  ierr = MatDestroy(&(*bv)->B);CHKERRQ(ierr);
-  ierr = MatDestroy(&(*bv)->C);CHKERRQ(ierr);
+  ierr = VecDestroy(&(*bv)->omega);CHKERRQ(ierr);
+  ierr = MatDestroy(&(*bv)->Acreate);CHKERRQ(ierr);
+  ierr = MatDestroy(&(*bv)->Aget);CHKERRQ(ierr);
+  ierr = MatDestroy(&(*bv)->Abuffer);CHKERRQ(ierr);
+  ierr = PetscRandomDestroy(&(*bv)->rand);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(bv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVCreate"
 /*@
    BVCreate - Creates a basis vectors context.
 
@@ -185,6 +183,8 @@ PetscErrorCode BVCreate(MPI_Comm comm,BV *newbv)
   bv->vmm          = BV_MATMULT_MAT;
 
   bv->Bx           = NULL;
+  bv->buffer       = NULL;
+  bv->Abuffer      = NULL;
   bv->xid          = 0;
   bv->xstate       = 0;
   bv->cv[0]        = NULL;
@@ -198,12 +198,21 @@ PetscErrorCode BVCreate(MPI_Comm comm,BV *newbv)
   bv->h            = NULL;
   bv->c            = NULL;
   bv->omega        = NULL;
-  bv->B            = NULL;
-  bv->C            = NULL;
-  bv->Aid          = 0;
   bv->defersfo     = PETSC_FALSE;
   bv->cached       = NULL;
   bv->bvstate      = 0;
+  bv->L            = NULL;
+  bv->R            = NULL;
+  bv->lstate       = 0;
+  bv->rstate       = 0;
+  bv->lsplit       = 0;
+  bv->issplit      = 0;
+  bv->splitparent  = NULL;
+  bv->rand         = NULL;
+  bv->rrandom      = PETSC_FALSE;
+  bv->Acreate      = NULL;
+  bv->Aget         = NULL;
+  bv->cuda         = PETSC_FALSE;
   bv->work         = NULL;
   bv->lwork        = 0;
   bv->data         = NULL;
@@ -212,8 +221,48 @@ PetscErrorCode BVCreate(MPI_Comm comm,BV *newbv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVInsertVec"
+/*@
+   BVCreateFromMat - Creates a basis vectors object from a dense Mat object.
+
+   Collective on Mat
+
+   Input Parameter:
+.  A - a dense tall-skinny matrix
+
+   Output Parameter:
+.  bv - the new basis vectors context
+
+   Notes:
+   The matrix values are copied to the BV data storage, memory is not shared.
+
+   The communicator of the BV object will be the same as A, and so will be
+   the dimensions.
+
+   Level: intermediate
+
+.seealso: BVCreate(), BVDestroy(), BVCreateMat()
+@*/
+PetscErrorCode BVCreateFromMat(Mat A,BV *bv)
+{
+  PetscErrorCode ierr;
+  PetscBool      match;
+  PetscInt       n,N,k;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(A,MAT_CLASSID,1);
+  ierr = PetscObjectTypeCompareAny((PetscObject)A,&match,MATSEQDENSE,MATMPIDENSE,"");CHKERRQ(ierr);
+  if (!match) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_SUP,"Matrix must be of type dense");
+
+  ierr = MatGetSize(A,&N,&k);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(A,&n,NULL);CHKERRQ(ierr);
+  ierr = BVCreate(PetscObjectComm((PetscObject)A),bv);CHKERRQ(ierr);
+  ierr = BVSetSizes(*bv,n,N,k);CHKERRQ(ierr);
+
+  (*bv)->Acreate = A;
+  ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    BVInsertVec - Insert a vector into the specified column.
 
@@ -254,8 +303,6 @@ PetscErrorCode BVInsertVec(BV V,PetscInt j,Vec w)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVInsertVecs"
 /*@
    BVInsertVecs - Insert a set of vectors into the specified columns.
 
@@ -295,7 +342,7 @@ PetscErrorCode BVInsertVecs(BV V,PetscInt s,PetscInt *m,Vec *W,PetscBool orth)
   PetscValidPointer(m,3);
   PetscValidLogicalCollectiveInt(V,*m,3);
   if (!*m) PetscFunctionReturn(0);
-  if (*m<0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Number of vectors (given %D) cannot be negative",*m);
+  if (*m<0) SETERRQ1(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Number of vectors (given %D) cannot be negative",*m);
   PetscValidPointer(W,4);
   PetscValidHeaderSpecific(*W,VEC_CLASSID,4);
   PetscValidLogicalCollectiveBool(V,orth,5);
@@ -329,8 +376,6 @@ PetscErrorCode BVInsertVecs(BV V,PetscInt s,PetscInt *m,Vec *W,PetscBool orth)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVInsertConstraints"
 /*@
    BVInsertConstraints - Insert a set of vectors as constraints.
 
@@ -376,12 +421,13 @@ PetscErrorCode BVInsertConstraints(BV V,PetscInt *nc,Vec *C)
   PetscValidPointer(nc,2);
   PetscValidLogicalCollectiveInt(V,*nc,2);
   if (!*nc) PetscFunctionReturn(0);
-  if (*nc<0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Number of constraints (given %D) cannot be negative",*nc);
+  if (*nc<0) SETERRQ1(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Number of constraints (given %D) cannot be negative",*nc);
   PetscValidPointer(C,3);
   PetscValidHeaderSpecific(*C,VEC_CLASSID,3);
   PetscValidType(V,1);
   BVCheckSizes(V,1);
   PetscCheckSameComm(V,1,*C,3);
+  if (V->issplit) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_WRONGSTATE,"Operation not permitted for a BV obtained from BVGetSplit");
   if (V->nc) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_WRONGSTATE,"Constraints already present in this BV object");
   if (V->ci[0]!=-1 || V->ci[1]!=-1) SETERRQ(PetscObjectComm((PetscObject)V),PETSC_ERR_SUP,"Cannot call BVInsertConstraints after BVGetColumn");
 
@@ -396,8 +442,6 @@ PetscErrorCode BVInsertConstraints(BV V,PetscInt *nc,Vec *C)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVSetOptionsPrefix"
 /*@C
    BVSetOptionsPrefix - Sets the prefix used for searching for all
    BV options in the database.
@@ -427,8 +471,6 @@ PetscErrorCode BVSetOptionsPrefix(BV bv,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVAppendOptionsPrefix"
 /*@C
    BVAppendOptionsPrefix - Appends to the prefix used for searching for all
    BV options in the database.
@@ -458,8 +500,6 @@ PetscErrorCode BVAppendOptionsPrefix(BV bv,const char *prefix)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVGetOptionsPrefix"
 /*@C
    BVGetOptionsPrefix - Gets the prefix used for searching for all
    BV options in the database.
@@ -472,7 +512,8 @@ PetscErrorCode BVAppendOptionsPrefix(BV bv,const char *prefix)
    Output Parameters:
 .  prefix - pointer to the prefix string used, is returned
 
-   Notes: On the Fortran side, the user should pass in a string 'prefix' of
+   Note:
+   On the Fortran side, the user should pass in a string 'prefix' of
    sufficient length to hold the prefix.
 
    Level: advanced
@@ -490,8 +531,6 @@ PetscErrorCode BVGetOptionsPrefix(BV bv,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVView_Default"
 static PetscErrorCode BVView_Default(BV bv,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -511,7 +550,7 @@ static PetscErrorCode BVView_Default(BV bv,PetscViewer viewer)
     ierr = PetscObjectGetName((PetscObject)bv,&bvname);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"%s=[];\n",bvname);CHKERRQ(ierr);
   }
-  for (j=bv->nc;j<bv->nc+bv->m;j++) {
+  for (j=-bv->nc;j<bv->m;j++) {
     ierr = BVGetColumn(bv,j,&v);CHKERRQ(ierr);
     ierr = VecView(v,viewer);CHKERRQ(ierr);
     if (ismatlab) {
@@ -523,8 +562,6 @@ static PetscErrorCode BVView_Default(BV bv,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVView"
 /*@C
    BVView - Prints the BV data structure.
 
@@ -553,10 +590,10 @@ PetscErrorCode BVView(BV bv,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
   PetscBool         isascii;
+  PetscInt          tabs;
   PetscViewerFormat format;
   const char        *orthname[2] = {"classical","modified"};
   const char        *refname[3] = {"if needed","never","always"};
-  const char        *borthname[2] = {"Gram-Schmidt","Cholesky"};
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
@@ -567,30 +604,31 @@ PetscErrorCode BVView(BV bv,PetscViewer viewer)
 
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
+    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
+    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)bv)->tablevel);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)bv,viewer);CHKERRQ(ierr);
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
-      ierr = PetscViewerASCIIPrintf(viewer,"%D columns of global length %D\n",bv->m,bv->N);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  %D columns of global length %D%s\n",bv->m,bv->N,bv->cuda?" (CUDA)":"");CHKERRQ(ierr);
       if (bv->nc>0) {
-        ierr = PetscViewerASCIIPrintf(viewer,"number of constraints: %D\n",bv->nc);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"  number of constraints: %D\n",bv->nc);CHKERRQ(ierr);
       }
-      ierr = PetscViewerASCIIPrintf(viewer,"vector orthogonalization method: %s Gram-Schmidt\n",orthname[bv->orthog_type]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  vector orthogonalization method: %s Gram-Schmidt\n",orthname[bv->orthog_type]);CHKERRQ(ierr);
       switch (bv->orthog_ref) {
         case BV_ORTHOG_REFINE_IFNEEDED:
-          ierr = PetscViewerASCIIPrintf(viewer,"orthogonalization refinement: %s (eta: %g)\n",refname[bv->orthog_ref],(double)bv->orthog_eta);CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer,"  orthogonalization refinement: %s (eta: %g)\n",refname[bv->orthog_ref],(double)bv->orthog_eta);CHKERRQ(ierr);
           break;
         case BV_ORTHOG_REFINE_NEVER:
         case BV_ORTHOG_REFINE_ALWAYS:
-          ierr = PetscViewerASCIIPrintf(viewer,"orthogonalization refinement: %s\n",refname[bv->orthog_ref]);CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer,"  orthogonalization refinement: %s\n",refname[bv->orthog_ref]);CHKERRQ(ierr);
           break;
       }
-      ierr = PetscViewerASCIIPrintf(viewer,"block orthogonalization method: %s\n",borthname[bv->orthog_block]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  block orthogonalization method: %s\n",BVOrthogBlockTypes[bv->orthog_block]);CHKERRQ(ierr);
       if (bv->matrix) {
         if (bv->indef) {
-          ierr = PetscViewerASCIIPrintf(viewer,"indefinite inner product\n");CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer,"  indefinite inner product\n");CHKERRQ(ierr);
         } else {
-          ierr = PetscViewerASCIIPrintf(viewer,"non-standard inner product\n");CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer,"  non-standard inner product\n");CHKERRQ(ierr);
         }
         ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
         ierr = MatView(bv->matrix,viewer);CHKERRQ(ierr);
@@ -598,30 +636,32 @@ PetscErrorCode BVView(BV bv,PetscViewer viewer)
       }
       switch (bv->vmm) {
         case BV_MATMULT_VECS:
-          ierr = PetscViewerASCIIPrintf(viewer,"doing matmult as matrix-vector products\n");CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer,"  doing matmult as matrix-vector products\n");CHKERRQ(ierr);
           break;
         case BV_MATMULT_MAT:
-          ierr = PetscViewerASCIIPrintf(viewer,"doing matmult as a single matrix-matrix product\n");CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer,"  doing matmult as a single matrix-matrix product\n");CHKERRQ(ierr);
           break;
         case BV_MATMULT_MAT_SAVE:
-          ierr = PetscViewerASCIIPrintf(viewer,"doing matmult as a single matrix-matrix product, saving aux matrices\n");CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer,"  mat_save is deprecated, use mat\n");CHKERRQ(ierr);
           break;
       }
+      if (bv->rrandom) {
+        ierr = PetscViewerASCIIPrintf(viewer,"  generating random vectors independent of the number of processes\n");CHKERRQ(ierr);
+      }
+      if (bv->ops->view) { ierr = (*bv->ops->view)(bv,viewer);CHKERRQ(ierr); }
     } else {
       if (bv->ops->view) { ierr = (*bv->ops->view)(bv,viewer);CHKERRQ(ierr); }
       else { ierr = BVView_Default(bv,viewer);CHKERRQ(ierr); }
     }
-    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
+    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   } else {
     ierr = (*bv->ops->view)(bv,viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVRegister"
 /*@C
-   BVRegister - Adds a new storage format to de BV package.
+   BVRegister - Adds a new storage format to the BV package.
 
    Not collective
 
@@ -642,12 +682,11 @@ PetscErrorCode BVRegister(const char *name,PetscErrorCode (*function)(BV))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = BVInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&BVList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BVAllocateWork_Private"
 PetscErrorCode BVAllocateWork_Private(BV bv,PetscInt s)
 {
   PetscErrorCode ierr;
