@@ -1,29 +1,16 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2015, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/dsimpl.h>
 #include <slepcblaslapack.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "DSAllocate_HEP"
 PetscErrorCode DSAllocate_HEP(DS ds,PetscInt ld)
 {
   PetscErrorCode ierr;
@@ -40,17 +27,17 @@ PetscErrorCode DSAllocate_HEP(DS ds,PetscInt ld)
 
 /*   0       l           k                 n-1
     -----------------------------------------
-    |*       ·           ·                  |
-    |  *     ·           ·                  |
-    |    *   ·           ·                  |
-    |      * ·           ·                  |
-    |· · · · o           o                  |
+    |*       .           .                  |
+    |  *     .           .                  |
+    |    *   .           .                  |
+    |      * .           .                  |
+    |. . . . o           o                  |
     |          o         o                  |
     |            o       o                  |
     |              o     o                  |
     |                o   o                  |
     |                  o o                  |
-    |· · · · o o o o o o o x                |
+    |. . . . o o o o o o o x                |
     |                    x x x              |
     |                      x x x            |
     |                        x x x          |
@@ -63,8 +50,6 @@ PetscErrorCode DSAllocate_HEP(DS ds,PetscInt ld)
     -----------------------------------------
 */
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSwitchFormat_HEP"
 static PetscErrorCode DSSwitchFormat_HEP(DS ds,PetscBool tocompact)
 {
   PetscErrorCode ierr;
@@ -80,7 +65,7 @@ static PetscErrorCode DSSwitchFormat_HEP(DS ds,PetscBool tocompact)
       T[i+ld] = PetscRealPart(A[k+i*ld]);
     }
     for (i=k;i<n-1;i++) {
-      T[i] = PetscRealPart(A[i+i*ld]);
+      T[i]    = PetscRealPart(A[i+i*ld]);
       T[i+ld] = PetscRealPart(A[i+1+i*ld]);
     }
     T[n-1] = PetscRealPart(A[n-1+(n-1)*ld]);
@@ -94,8 +79,8 @@ static PetscErrorCode DSSwitchFormat_HEP(DS ds,PetscBool tocompact)
     }
     A[k+k*ld] = T[k];
     for (i=k+1;i<n;i++) {
-      A[i+i*ld] = T[i];
-      A[i-1+i*ld] = T[i-1+ld];
+      A[i+i*ld]     = T[i];
+      A[i-1+i*ld]   = T[i-1+ld];
       A[i+(i-1)*ld] = T[i-1+ld];
     }
     if (ds->extrarow) A[n+(n-1)*ld] = T[n-1+ld];
@@ -103,8 +88,6 @@ static PetscErrorCode DSSwitchFormat_HEP(DS ds,PetscBool tocompact)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSView_HEP"
 PetscErrorCode DSView_HEP(DS ds,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -125,9 +108,7 @@ PetscErrorCode DSView_HEP(DS ds,PetscViewer viewer)
     if (ds->bs>1) {
       ierr = PetscViewerASCIIPrintf(viewer,"block size: %D\n",ds->bs);CHKERRQ(ierr);
     }
-    if (ds->method>=nmeth) {
-      ierr = PetscViewerASCIIPrintf(viewer,"solving the problem with: INVALID METHOD\n");CHKERRQ(ierr);
-    } else {
+    if (ds->method<nmeth) {
       ierr = PetscViewerASCIIPrintf(viewer,"solving the problem with: %s\n",methodname[ds->method]);CHKERRQ(ierr);
     }
     PetscFunctionReturn(0);
@@ -140,14 +121,14 @@ PetscErrorCode DSView_HEP(DS ds,PetscViewer viewer)
       ierr = PetscViewerASCIIPrintf(viewer,"zzz = zeros(%D,3);\n",3*ds->n);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPrintf(viewer,"zzz = [\n");CHKERRQ(ierr);
       for (i=0;i<ds->n;i++) {
-        ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",i+1,i+1,*(ds->rmat[DS_MAT_T]+i));CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",i+1,i+1,(double)*(ds->rmat[DS_MAT_T]+i));CHKERRQ(ierr);
       }
       for (i=0;i<rows-1;i++) {
         r = PetscMax(i+2,ds->k+1);
         c = i+1;
-        ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",r,c,*(ds->rmat[DS_MAT_T]+ds->ld+i));CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",r,c,(double)*(ds->rmat[DS_MAT_T]+ds->ld+i));CHKERRQ(ierr);
         if (i<ds->n-1 && ds->k<ds->n) { /* do not print vertical arrow when k=n */
-          ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",c,r,*(ds->rmat[DS_MAT_T]+ds->ld+i));CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",c,r,(double)*(ds->rmat[DS_MAT_T]+ds->ld+i));CHKERRQ(ierr);
         }
       }
       ierr = PetscViewerASCIIPrintf(viewer,"];\n%s = spconvert(zzz);\n",DSMatName[DS_MAT_T]);CHKERRQ(ierr);
@@ -159,7 +140,7 @@ PetscErrorCode DSView_HEP(DS ds,PetscViewer viewer)
           else if (i==j+1 && i>ds->k) value = *(ds->rmat[DS_MAT_T]+ds->ld+i-1);
           else if (i+1==j && j>ds->k) value = *(ds->rmat[DS_MAT_T]+ds->ld+j-1);
           else value = 0.0;
-          ierr = PetscViewerASCIIPrintf(viewer," %18.16e ",value);CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer," %18.16e ",(double)value);CHKERRQ(ierr);
         }
         ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
       }
@@ -175,8 +156,6 @@ PetscErrorCode DSView_HEP(DS ds,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSVectors_HEP"
 PetscErrorCode DSVectors_HEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
 {
   PetscScalar    *Q = ds->mat[DS_MAT_Q];
@@ -202,11 +181,11 @@ PetscErrorCode DSVectors_HEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
           for (i=0;i<ds->n;i++) *(ds->mat[mat]+i+i*ld) = 1.0;
         }
       }
-      if (rnorm) *rnorm = PetscAbsScalar(Q[ds->n-1+(*j)*ld]);
+      if (rnorm && j) *rnorm = PetscAbsScalar(Q[ds->n-1+(*j)*ld]);
       break;
     case DS_MAT_U:
     case DS_MAT_VT:
-      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not implemented yet");
+      SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"Not implemented yet");
       break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid mat parameter");
@@ -214,29 +193,6 @@ PetscErrorCode DSVectors_HEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSNormalize_HEP"
-PetscErrorCode DSNormalize_HEP(DS ds,DSMatType mat,PetscInt col)
-{
-  PetscFunctionBegin;
-  switch (mat) {
-    case DS_MAT_X:
-    case DS_MAT_Y:
-    case DS_MAT_Q:
-      /* All the matrices resulting from DSVectors and DSSolve are already normalized */
-      break;
-    case DS_MAT_U:
-    case DS_MAT_VT:
-      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not implemented yet");
-      break;
-    default:
-      SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid mat parameter");
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "ArrowTridiag"
 /*
   ARROWTRIDIAG reduces a symmetric arrowhead matrix of the form
 
@@ -300,7 +256,7 @@ static PetscErrorCode ArrowTridiag(PetscBLASInt n,PetscReal *d,PetscReal *e,Pets
 
     /* Eliminate entry e(j) by a rotation in the planes (j,j+1) */
     temp = e[j+1];
-    PetscStackCallBLAS("LAPACKlartg",LAPACKlartg_(&temp,&e[j],&c,&s,&e[j+1]));
+    PetscStackCallBLAS("LAPACKlartg",LAPACKREALlartg_(&temp,&e[j],&c,&s,&e[j+1]));
     s = -s;
 
     /* Apply rotation to diagonal elements */
@@ -311,14 +267,14 @@ static PetscErrorCode ArrowTridiag(PetscBLASInt n,PetscReal *d,PetscReal *e,Pets
 
     /* Apply rotation to Q */
     j2 = j+2;
-    PetscStackCallBLAS("BLASrot",BLASrot_(&j2,Q+j*ld,&one,Q+(j+1)*ld,&one,&c,&s));
+    PetscStackCallBLAS("BLASrot",BLASMIXEDrot_(&j2,Q+j*ld,&one,Q+(j+1)*ld,&one,&c,&s));
 
     /* Chase newly introduced off-diagonal entry to the top left corner */
     for (i=j-1;i>=0;i--) {
       off  = -s*e[i];
       e[i] = c*e[i];
       temp = e[i+1];
-      PetscStackCallBLAS("LAPACKlartg",LAPACKlartg_(&temp,&off,&c,&s,&e[i+1]));
+      PetscStackCallBLAS("LAPACKlartg",LAPACKREALlartg_(&temp,&off,&c,&s,&e[i+1]));
       s = -s;
       temp = (d[i]-d[i+1])*s - 2.0*c*e[i];
       p = s*temp;
@@ -326,15 +282,13 @@ static PetscErrorCode ArrowTridiag(PetscBLASInt n,PetscReal *d,PetscReal *e,Pets
       d[i] -= p;
       e[i] = -e[i] - c*temp;
       j2 = j+2;
-      PetscStackCallBLAS("BLASrot",BLASrot_(&j2,Q+i*ld,&one,Q+(i+1)*ld,&one,&c,&s));
+      PetscStackCallBLAS("BLASrot",BLASMIXEDrot_(&j2,Q+i*ld,&one,Q+(i+1)*ld,&one,&c,&s));
     }
   }
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSIntermediate_HEP"
 /*
    Reduce to tridiagonal form by means of ArrowTridiag.
 */
@@ -380,12 +334,12 @@ static PetscErrorCode DSIntermediate_HEP(DS ds)
       work = ds->work+ld;
       lwork = ld*ld;
       PetscStackCallBLAS("LAPACKsytrd",LAPACKsytrd_("L",&n3,Q+off,&ld,d+l,e+l,tau,work,&lwork,&info));
-      if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xSYTRD %d",info);
+      SlepcCheckLapackInfo("sytrd",info);
       PetscStackCallBLAS("LAPACKorgtr",LAPACKorgtr_("L",&n3,Q+off,&ld,tau,work,&lwork,&info));
-      if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xORGTR %d",info);
+      SlepcCheckLapackInfo("orgtr",info);
     } else {
       /* copy tridiagonal to d,e */
-      for (i=l;i<n;i++) d[i] = PetscRealPart(A[i+i*ld]);
+      for (i=l;i<n;i++)   d[i] = PetscRealPart(A[i+i*ld]);
       for (i=l;i<n-1;i++) e[i] = PetscRealPart(A[(i+1)+i*ld]);
     }
   }
@@ -393,8 +347,6 @@ static PetscErrorCode DSIntermediate_HEP(DS ds)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSort_HEP"
 PetscErrorCode DSSort_HEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
 {
   PetscErrorCode ierr;
@@ -423,8 +375,6 @@ PetscErrorCode DSSort_HEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSUpdateExtraRow_HEP"
 PetscErrorCode DSUpdateExtraRow_HEP(DS ds)
 {
   PetscErrorCode ierr;
@@ -456,8 +406,6 @@ PetscErrorCode DSUpdateExtraRow_HEP(DS ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSolve_HEP_QR"
 PetscErrorCode DSSolve_HEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
 #if defined(PETSC_MISSING_LAPACK_STEQR)
@@ -471,7 +419,7 @@ PetscErrorCode DSSolve_HEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
   PetscReal      *d,*e;
 
   PetscFunctionBegin;
-  if (ds->bs>1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"This method is not prepared for bs>1");
+  if (ds->bs>1) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"This method is not prepared for bs>1");
   ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->l,&l);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
@@ -492,7 +440,7 @@ PetscErrorCode DSSolve_HEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   ierr = DSAllocateWork_Private(ds,0,2*ld,0);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKsteqr",LAPACKsteqr_("V",&n3,d+l,e+l,Q+off,&ld,ds->rwork,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xSTEQR %d",info);
+  SlepcCheckLapackInfo("steqr",info);
   for (i=l;i<n;i++) wr[i] = d[i];
 
   /* Create diagonal matrix as a result */
@@ -511,8 +459,6 @@ PetscErrorCode DSSolve_HEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSolve_HEP_MRRR"
 PetscErrorCode DSSolve_HEP_MRRR(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
 #if defined(SLEPC_MISSING_LAPACK_STEVR)
@@ -530,7 +476,7 @@ PetscErrorCode DSSolve_HEP_MRRR(DS ds,PetscScalar *wr,PetscScalar *wi)
 #endif
 
   PetscFunctionBegin;
-  if (ds->bs>1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"This method is not prepared for bs>1");
+  if (ds->bs>1) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"This method is not prepared for bs>1");
   ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->l,&l);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
@@ -568,7 +514,7 @@ PetscErrorCode DSSolve_HEP_MRRR(DS ds,PetscScalar *wr,PetscScalar *wi)
 #else
   PetscStackCallBLAS("LAPACKstevr",LAPACKstevr_("V","A",&n3,d+l,e+l,&vl,&vu,&il,&iu,&abstol,&m,wr+l,Q+off,&ld,isuppz,ds->rwork,&lwork,ds->iwork,&liwork,&info));
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack DSTEVR %d",info);
+  SlepcCheckLapackInfo("stevr",info);
 #if defined(PETSC_USE_COMPLEX)
   for (i=l;i<n;i++)
     for (j=l;j<n;j++)
@@ -596,13 +542,11 @@ PetscErrorCode DSSolve_HEP_MRRR(DS ds,PetscScalar *wr,PetscScalar *wi)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSolve_HEP_DC"
 PetscErrorCode DSSolve_HEP_DC(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(SLEPC_MISSING_LAPACK_STEDC) || defined(SLEPC_MISSING_LAPACK_ORMTR)
+#if defined(SLEPC_MISSING_LAPACK_STEDC)
   PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"STEDC/ORMTR - Lapack routine is unavailable");
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"STEDC - Lapack routine is unavailable");
 #else
   PetscErrorCode ierr;
   PetscInt       i;
@@ -615,7 +559,7 @@ PetscErrorCode DSSolve_HEP_DC(DS ds,PetscScalar *wr,PetscScalar *wi)
 #endif
 
   PetscFunctionBegin;
-  if (ds->bs>1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"This method is not prepared for bs>1");
+  if (ds->bs>1) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"This method is not prepared for bs>1");
   ierr = PetscBLASIntCast(ds->l,&l);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->n-ds->l,&n1);CHKERRQ(ierr);
@@ -644,7 +588,7 @@ PetscErrorCode DSSolve_HEP_DC(DS ds,PetscScalar *wr,PetscScalar *wi)
   for (j=ds->l;j<ds->n;j++)
     for (i=0;i<ds->l;i++) Q[i+j*ld] = 0.0;
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xSTEDC %d",info);
+  SlepcCheckLapackInfo("stedc",info);
   for (i=l;i<ds->n;i++) wr[i] = d[i];
 
   /* Create diagonal matrix as a result */
@@ -664,8 +608,6 @@ PetscErrorCode DSSolve_HEP_DC(DS ds,PetscScalar *wr,PetscScalar *wi)
 }
 
 #if !defined(PETSC_USE_COMPLEX)
-#undef __FUNCT__
-#define __FUNCT__ "DSSolve_HEP_BDC"
 PetscErrorCode DSSolve_HEP_BDC(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
   PetscErrorCode ierr;
@@ -674,8 +616,8 @@ PetscErrorCode DSSolve_HEP_BDC(DS ds,PetscScalar *wr,PetscScalar *wi)
   PetscReal      *D,*E,*d,*e,tol=PETSC_MACHINE_EPSILON/2,tau1=1e-16,tau2=1e-18,*rwork,mingap;
 
   PetscFunctionBegin;
-  if (ds->l>0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"This method is not prepared for l>1");
-  if (ds->compact) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not implemented for compact storage");
+  if (ds->l>0) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"This method is not prepared for l>1");
+  if (ds->compact) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"Not implemented for compact storage");
   ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->bs,&bs);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
@@ -733,12 +675,10 @@ PetscErrorCode DSSolve_HEP_BDC(DS ds,PetscScalar *wr,PetscScalar *wi)
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "DSTruncate_HEP"
 PetscErrorCode DSTruncate_HEP(DS ds,PetscInt n)
 {
-  PetscInt       i,ld=ds->ld,l=ds->l;
-  PetscScalar    *A;
+  PetscInt    i,ld=ds->ld,l=ds->l;
+  PetscScalar *A;
 
   PetscFunctionBegin;
   if (ds->state==DS_STATE_CONDENSED) ds->t = ds->n;
@@ -752,13 +692,65 @@ PetscErrorCode DSTruncate_HEP(DS ds,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSCond_HEP"
+PetscErrorCode DSSynchronize_HEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
+{
+  PetscErrorCode ierr;
+  PetscInt       ld=ds->ld,l=ds->l,k=0,kr=0;
+  PetscMPIInt    n,rank,off=0,size,ldn,ld3;
+
+  PetscFunctionBegin;
+  if (ds->compact) kr = 3*ld;
+  else k = (ds->n-l)*ld;
+  if (ds->state>DS_STATE_RAW) k += (ds->n-l)*ld;
+  if (eigr) k += (ds->n-l); 
+  if (eigi) k += (ds->n-l); 
+  ierr = DSAllocateWork_Private(ds,k+kr,0,0);CHKERRQ(ierr);
+  ierr = PetscMPIIntCast(k*sizeof(PetscScalar)+kr*sizeof(PetscReal),&size);CHKERRQ(ierr);
+  ierr = PetscMPIIntCast(ds->n-l,&n);CHKERRQ(ierr);
+  ierr = PetscMPIIntCast(ld*(ds->n-l),&ldn);CHKERRQ(ierr);
+  ierr = PetscMPIIntCast(ld*3,&ld3);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)ds),&rank);CHKERRQ(ierr);
+  if (!rank) {
+    if (ds->compact) {
+      ierr = MPI_Pack(ds->rmat[DS_MAT_T],ld3,MPIU_REAL,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    } else {
+      ierr = MPI_Pack(ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (ds->state>DS_STATE_RAW) {
+      ierr = MPI_Pack(ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (eigr) {
+      ierr = MPI_Pack(eigr+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (eigi) {
+      ierr = MPI_Pack(eigi+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+  }
+  ierr = MPI_Bcast(ds->work,size,MPI_BYTE,0,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+  if (rank) {
+    if (ds->compact) {
+      ierr = MPI_Unpack(ds->work,size,&off,ds->rmat[DS_MAT_T],ld3,MPIU_REAL,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    } else {
+      ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (ds->state>DS_STATE_RAW) {
+      ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (eigr) {
+      ierr = MPI_Unpack(ds->work,size,&off,eigr+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (eigi) {
+      ierr = MPI_Unpack(ds->work,size,&off,eigi+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode DSCond_HEP(DS ds,PetscReal *cond)
 {
-#if defined(PETSC_MISSING_LAPACK_GETRF) || defined(SLEPC_MISSING_LAPACK_GETRI) || defined(SLEPC_MISSING_LAPACK_LANGE) || defined(SLEPC_MISSING_LAPACK_LANHS)
+#if defined(PETSC_MISSING_LAPACK_GETRF) || defined(PETSC_MISSING_LAPACK_GETRI) || defined(SLEPC_MISSING_LAPACK_LANGE)
   PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRF/GETRI/LANGE/LANHS - Lapack routines are unavailable");
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRF/GETRI/LANGE - Lapack routines are unavailable");
 #else
   PetscErrorCode ierr;
   PetscScalar    *work;
@@ -788,9 +780,9 @@ PetscErrorCode DSCond_HEP(DS ds,PetscReal *cond)
 
   /* norm of inv(A) */
   PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n,&n,A,&ld,ipiv,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGETRF %d",info);
+  SlepcCheckLapackInfo("getrf",info);
   PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&n,A,&ld,ipiv,work,&lwork,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGETRI %d",info);
+  SlepcCheckLapackInfo("getri",info);
   hin = LAPACKlange_("I",&n,&n,A,&ld,rwork);
 
   *cond = hn*hin;
@@ -798,11 +790,9 @@ PetscErrorCode DSCond_HEP(DS ds,PetscReal *cond)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSTranslateRKS_HEP"
 PetscErrorCode DSTranslateRKS_HEP(DS ds,PetscScalar alpha)
 {
-#if defined(PETSC_MISSING_LAPACK_GEQRF) || defined(SLEPC_MISSING_LAPACK_ORGQR)
+#if defined(PETSC_MISSING_LAPACK_GEQRF) || defined(PETSC_MISSING_LAPACK_ORGQR)
   PetscFunctionBegin;
   SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQRF/ORGQR - Lapack routines are unavailable");
 #else
@@ -821,39 +811,50 @@ PetscErrorCode DSTranslateRKS_HEP(DS ds,PetscScalar alpha)
   A  = ds->mat[DS_MAT_A];
   Q  = ds->mat[DS_MAT_Q];
   R  = ds->mat[DS_MAT_W];
-  /* Copy I+alpha*A */
+
+  /* copy I+alpha*A */
   ierr = PetscMemzero(Q,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
   ierr = PetscMemzero(R,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
   for (i=0;i<k;i++) {
     Q[i+i*ld] = 1.0 + alpha*A[i+i*ld];
     Q[k+i*ld] = alpha*A[k+i*ld];
   }
-  /* Compute qr */
+
+  /* compute qr */
   ierr = PetscBLASIntCast(k+1,&n1);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(k,&n0);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKgeqrf",LAPACKgeqrf_(&n1,&n0,Q,&ld,tau,work,&lwork,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGEQRF %d",info);
-  /* Copy R from Q */
+  SlepcCheckLapackInfo("geqrf",info);
+
+  /* copy R from Q */
   for (j=0;j<k;j++)
     for (i=0;i<=j;i++)
       R[i+j*ld] = Q[i+j*ld];
-  /* Compute orthogonal matrix in Q */
+
+  /* compute orthogonal matrix in Q */
   PetscStackCallBLAS("LAPACKorgqr",LAPACKorgqr_(&n1,&n1,&n0,Q,&ld,tau,work,&lwork,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xORGQR %d",info);
-  /* Compute the updated matrix of projected problem */
+  SlepcCheckLapackInfo("orgqr",info);
+
+  /* compute the updated matrix of projected problem */
   for (j=0;j<k;j++)
     for (i=0;i<k+1;i++)
       A[j*ld+i] = Q[i*ld+j];
   alpha = -1.0/alpha;
   PetscStackCallBLAS("BLAStrsm",BLAStrsm_("R","U","N","N",&n1,&n0,&alpha,R,&ld,A,&ld));
   for (i=0;i<k;i++)
-    A[ld*i+i]-=alpha;
+    A[ld*i+i] -= alpha;
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSCreate_HEP"
+PetscErrorCode DSHermitian_HEP(DS ds,DSMatType m,PetscBool *flg)
+{
+  PetscFunctionBegin;
+  if (m==DS_MAT_A && !ds->extrarow) *flg = PETSC_TRUE;
+  else *flg = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
 PETSC_EXTERN PetscErrorCode DSCreate_HEP(DS ds)
 {
   PetscFunctionBegin;
@@ -867,11 +868,12 @@ PETSC_EXTERN PetscErrorCode DSCreate_HEP(DS ds)
   ds->ops->solve[3]      = DSSolve_HEP_BDC;
 #endif
   ds->ops->sort          = DSSort_HEP;
+  ds->ops->synchronize   = DSSynchronize_HEP;
   ds->ops->truncate      = DSTruncate_HEP;
   ds->ops->update        = DSUpdateExtraRow_HEP;
   ds->ops->cond          = DSCond_HEP;
   ds->ops->transrks      = DSTranslateRKS_HEP;
-  ds->ops->normalize     = DSNormalize_HEP;
+  ds->ops->hermitian     = DSHermitian_HEP;
   PetscFunctionReturn(0);
 }
 
